/***************************************************************************
                     configdlg.cpp  -  The settings dialog.
                     -------------------
begin                : Do Okt 16 14:35:18 CEST 2003
copyright            : (C) 2003 by Mark Kretschmann
email                :
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "configdlg.h"
#include "browserwidget.h"
#include "browserwin.h"
#include "Options1.h"
#include "Options2.h"
#include "Options3.h"
#include "Options4.h"
#include "playerapp.h"
#include "playlistwidget.h"

#include <qcheckbox.h>
#include <qcolor.h>
#include <qcombobox.h>
#include <qfont.h>
#include <qlabel.h>
#include <qpushbutton.h>
//#include <qsize.h>
#include <qspinbox.h>
#include <qvbox.h>

#include <kcolorbutton.h>
#include <kdialogbase.h>
#include <kfontdialog.h>
#include <kiconloader.h>
#include <klocale.h>


ConfigDlg::ConfigDlg() : KDialogBase( KDialogBase::IconList, i18n( "Options" ),
                                              KDialogBase::Ok | KDialogBase::Apply | KDialogBase::Cancel,
                                              KDialogBase::Ok, 0, 0, false )
{
    KIconLoader iconLoader;

    // PAGE: OPTIONS1 (misc) ===========================================
    QVBox *pVBox1 = addVBoxPage( i18n( "General" ) , i18n( "Configure general options" ),
                                 iconLoader.loadIcon( "misc", KIcon::NoGroup, KIcon::SizeMedium ) );

    m_pOptions1 = new Options1( pVBox1 );
    initOptions1();

    // PAGE: OPTIONS2 (fonts) ===========================================
    QVBox *pVBox2 = addVBoxPage( i18n( "Fonts" ) , i18n( "Configure fonts" ),
                                 iconLoader.loadIcon( "fonts", KIcon::NoGroup, KIcon::SizeMedium ) );

    m_pOptions2 = new Options2( pVBox2 );
    initOptions2();

    // PAGE: OPTIONS3 (colors) ==========================================
    QVBox *pVBox3 = addVBoxPage( i18n( "Colors" ) , i18n( "Configure Colors" ),
                                 iconLoader.loadIcon( "colors", KIcon::NoGroup, KIcon::SizeMedium ) );

    m_pOptions3 = new Options3( pVBox3 );
    initOptions3();
    
    QVBox *pVBox4 = addVBoxPage( i18n( "Playback" ), i18n( "Configure playback" ),
                                 iconLoader.loadIcon( "kmix", KIcon::NoGroup, KIcon::SizeMedium ) );
    
    m_pOptions4 = new Options4( pVBox4 );
    initOptions4();

    enableButtonApply( false );
    setInitialSize( QSize( 480, 430 ) );
}


ConfigDlg::~ConfigDlg()
{
}


// METHODS ======================================================================

void ConfigDlg::slotOk()
{
    slotSave();
    accept();
}


void ConfigDlg::slotApply()
{
    slotSave();
    enableButtonApply( false );
}


void ConfigDlg::slotCancel()
{
    reject();
}


void ConfigDlg::slotSave()
{
    saveOptions1();
    saveOptions2();
    saveOptions3();
    saveOptions4();
}


void ConfigDlg::slotChanged()
{
    enableButtonApply( true );
}


// INIT ========================================================================

void ConfigDlg::initOptions1()
{
    m_pOptions1->checkBox1->setChecked( pApp->m_optSavePlaylist );
    m_pOptions1->checkBox7->setChecked( pApp->m_optConfirmClear );
    m_pOptions1->checkBox6->setChecked( pApp->m_optConfirmExit );
    m_pOptions1->checkBox4->setChecked( pApp->m_optReadMetaInfo );
    m_pOptions1->checkBox3->setChecked( pApp->m_optShowTrayIcon );
    m_pOptions1->checkBox5->setChecked( pApp->m_optHidePlaylistWindow );

    if ( pApp->m_optDropMode == "Ask" )
        m_pOptions1->comboBox1->setCurrentItem( 0 );
    if ( pApp->m_optDropMode == "Recursively" )
        m_pOptions1->comboBox1->setCurrentItem( 1 );
    if ( pApp->m_optDropMode == "NonRecursively" )
        m_pOptions1->comboBox1->setCurrentItem( 2 );

    connect( m_pOptions1->checkBox1, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->checkBox7, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->checkBox6, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->checkBox4, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->checkBox3, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->checkBox5, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->comboBox1, SIGNAL( activated( int ) ),
             this, SLOT( slotChanged() ) );
}


void ConfigDlg::initOptions2()
{
    m_optPlayerWidgetFont = pApp->m_optPlayerWidgetFont;
    m_optPlayerWidgetScrollFont = pApp->m_optPlayerWidgetScrollFont;
    m_optBrowserWindowFont = pApp->m_optBrowserWindowFont;

    m_pOptions2->labelPlayerWidgetFont->setFont( m_optPlayerWidgetFont );
    m_pOptions2->labelPlayerWidgetScrollerFont->setFont( m_optPlayerWidgetScrollFont );
    m_pOptions2->labelBrowserWindowFont->setFont( m_optBrowserWindowFont );

    m_pOptions2->labelPlayerWidgetFont->setText( "FREE OFFER" );
    m_pOptions2->labelPlayerWidgetScrollerFont->setText( "ENLARGEMENT PILLS" );
    m_pOptions2->labelBrowserWindowFont->setText( "THIS IS NO SPAM" );

    connect( m_pOptions2->pushPlayerWidgetFont, SIGNAL( clicked() ),
             this, SLOT( slotChoosePlayerWidgetFont() ) );
    connect( m_pOptions2->pushPlayerWidgetScrollerFont, SIGNAL( clicked() ),
             this, SLOT( slotChoosePlayerWidgetScrollFont() ) );
    connect( m_pOptions2->pushBrowserWindowFont, SIGNAL( clicked() ),
             this, SLOT( slotChooseBrowserWindowFont() ) );
}


void ConfigDlg::initOptions3()
{
    m_optBrowserBgColor = pApp->m_optBrowserBgColor;
    m_optBrowserFgColor = pApp->m_optBrowserFgColor;

    m_pOptions3->chkBoxTransparency->setDisabled( true );
    m_pOptions3->chkBoxCustomColor->setChecked( pApp->m_optBrowserUseCustomColors );
    m_pOptions3->slotToggled( pApp->m_optBrowserUseCustomColors ); //must call as is only called on a state change and above may not change the state, so badly designed I know
    m_pOptions3->kColorButtonBg->setColor( m_optBrowserBgColor );
    m_pOptions3->kColorButtonFg->setColor( m_optBrowserFgColor );

    connect( m_pOptions3->chkBoxTransparency, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions3->chkBoxCustomColor, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions3->kColorButtonBg, SIGNAL( changed( const QColor& ) ),
             this, SLOT( slotChooseBrowserWinBgColor( const QColor& ) ) );
    connect( m_pOptions3->kColorButtonFg, SIGNAL( changed( const QColor& ) ),
             this, SLOT( slotChooseBrowserWinFgColor( const QColor& ) ) );
}


void ConfigDlg::initOptions4()
{
    m_pOptions4->checkBoxXFade->setChecked( pApp->m_optXFade );
    m_pOptions4->crossfadeLength->setValue( pApp->m_optXFadeLength );
    m_pOptions4->checkBoxSwMixerOnly->setChecked( pApp->m_optSoftwareMixerOnly );
    m_pOptions4->trackDelayLength ->setValue( pApp->m_optTrackDelay );
    
    // take care of enabling / disabling crossfade spin boxes on startup
    crossfadeChanged( m_pOptions4->checkBoxXFade->isChecked() ? 2 : 0 );
 
    connect( m_pOptions4->checkBoxXFade, SIGNAL( stateChanged( int ) ),
             this, SLOT( crossfadeChanged( int ) ) );
    connect( m_pOptions4->checkBoxSwMixerOnly, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
}

// SAVE ========================================================================

void ConfigDlg::saveOptions1()
{
    pApp->m_optSavePlaylist = m_pOptions1->checkBox1->isChecked();
    pApp->m_optConfirmClear = m_pOptions1->checkBox7->isChecked();
    pApp->m_optConfirmExit = m_pOptions1->checkBox6->isChecked();
    pApp->m_optReadMetaInfo = m_pOptions1->checkBox4->isChecked();
    pApp->m_optShowTrayIcon = m_pOptions1->checkBox3->isChecked();
    pApp->m_optHidePlaylistWindow = m_pOptions1->checkBox5->isChecked();

    emit sigShowTrayIcon( pApp->m_optShowTrayIcon );

    if( m_pOptions1->comboBox1->currentItem() == 0 )
            pApp->m_optDropMode = "Ask";
    if( m_pOptions1->comboBox1->currentItem() == 1 )
            pApp->m_optDropMode = "Recursively";
    if( m_pOptions1->comboBox1->currentItem() == 2 )
            pApp->m_optDropMode = "NonRecursively";
}


void ConfigDlg::saveOptions2()
{
    pApp->m_optPlayerWidgetFont = m_optPlayerWidgetFont;
    pApp->m_optPlayerWidgetScrollFont = m_optPlayerWidgetScrollFont;
    pApp->m_optBrowserWindowFont = m_optBrowserWindowFont;

    pApp->m_pBrowserWin->slotUpdateFonts();
}


void ConfigDlg::saveOptions3()
{
    QColor bg, fg;

    if( pApp->m_optBrowserUseCustomColors = m_pOptions3->chkBoxCustomColor->isChecked() )
    {
       fg = m_optBrowserFgColor;
       bg = m_optBrowserBgColor;

    } else {

       fg = pApp->m_fgColor;
       bg = pApp->m_bgColor;
    }

    pApp->m_optBrowserBgColor = bg;
    pApp->m_optBrowserFgColor = fg;

    pApp->setupColors();
}


void ConfigDlg::saveOptions4()
{
    pApp->m_optXFade = m_pOptions4->checkBoxXFade->isChecked();
    pApp->m_optXFadeLength = m_pOptions4->crossfadeLength->value();
    pApp->m_optSoftwareMixerOnly = m_pOptions4->checkBoxSwMixerOnly->isChecked();
    pApp->m_optTrackDelay = m_pOptions4->trackDelayLength->value();
}


// SLOTS ========================================================================

void ConfigDlg::slotChoosePlayerWidgetFont()
{
    if ( KFontDialog::getFont( m_optPlayerWidgetFont ) == QDialog::Accepted )
    {
        m_pOptions2->labelPlayerWidgetFont->setFont( m_optPlayerWidgetFont );
        slotChanged();
    }
}


void ConfigDlg::slotChoosePlayerWidgetScrollFont()
{
    if ( KFontDialog::getFont( m_optPlayerWidgetScrollFont ) == QDialog::Accepted )
    {
        m_pOptions2->labelPlayerWidgetScrollerFont->setFont( m_optPlayerWidgetScrollFont );
        slotChanged();
    }
}


void ConfigDlg::slotChooseBrowserWindowFont()
{
    if ( KFontDialog::getFont( m_optBrowserWindowFont ) == QDialog::Accepted )
    {
        m_pOptions2->labelBrowserWindowFont->setFont( m_optBrowserWindowFont );
        slotChanged();
    }
}

void ConfigDlg::slotChooseBrowserWinBgColor( const QColor & col )
{
    m_optBrowserBgColor = col;

    slotChanged();
}


void ConfigDlg::slotChooseBrowserWinFgColor( const QColor & col )
{
    m_optBrowserFgColor = col;

    slotChanged();
}


void ConfigDlg::crossfadeChanged( int state )
{
    m_pOptions4->crossfadeLengthLabel->setEnabled( state == 2 );
    m_pOptions4->crossfadeLength->setEnabled( state == 2 );

    m_pOptions4->trackDelayLabel->setEnabled( state == 0);
    m_pOptions4->trackDelayLength->setEnabled( state == 0);
    
    slotChanged();
}


#include "configdlg.moc"
